/*
 * Copyright (c) 2020, Max Klein
 *
 * This file is part of the tree-finder library, distributed under the terms of
 * the BSD 3 Clause license. The full license can be found in the LICENSE file.
 */
import { BehaviorSubject } from "rxjs";
import { Content } from "./content";
import { SortStates, sortContentRoot } from "./sort";
export class ContentsModel {
    constructor(root, options = {}) {
        this.crumbSubject = new BehaviorSubject([]);
        this.options = options;
        this.reset();
        if (root) {
            this.setRoot(root);
        }
    }
    reset(crumbIx) {
        this._contents = [];
        if (!crumbIx) {
            this._crumbs = [];
            this.crumbSubject.next(this._crumbs.map(x => x.name));
            return;
        }
        const rootContent = this._crumbs[crumbIx];
        this._crumbs = this._crumbs.slice(0, crumbIx);
        this.setRoot(rootContent.row);
    }
    setRoot(root) {
        this._sortStates = new SortStates();
        this._root = new Content(root);
        this._crumbs.push(this._root);
        this.crumbSubject.next(this._crumbs.map(x => x.name));
        this.initColumns();
        // fetch root's children and mark it as open
        this._root.open(this._options.doRefetch);
        // set initial sort
        this.sort();
    }
    initColumns() {
        var _a;
        const columns = (_a = this.options.columnNames) !== null && _a !== void 0 ? _a : Object.keys(this._root.row);
        this._columns = columns.filter(x => !["path", "getChildren"].includes(x));
        // build a columnName -> columnIx table; add 1 to account for leading "path" column
        this._ixByColumn = Object.fromEntries(this._columns.map((x, ix) => [x, ix + 1]));
    }
    // splice out the contents of the collapsed node and any expanded subnodes
    async collapse(rix) {
        var _a, _b;
        const content = this._contents[rix];
        let npop = (_b = (_a = content.children) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0;
        let check_ix = rix + 1 + npop;
        while (this._contents[check_ix++].row.path.length > content.row.path.length) {
            npop++;
        }
        this._contents.splice(rix + 1, npop);
        content.close();
    }
    async expand(rix) {
        const content = this._contents[rix];
        content.open(this._options.doRefetch);
        const [nodeContents, _] = sortContentRoot({ root: this._contents[rix], sortStates: this._sortStates });
        this._contents.splice(rix + 1, 0, ...nodeContents);
    }
    sort(props = {}) {
        const { col, multisort } = props;
        this._root.open(this._options.doRefetch);
        [this._contents, this._sortStates] = sortContentRoot({ root: this._root, sortStates: this._sortStates, col, multisort });
    }
    get columns() {
        return this._columns;
    }
    get contents() {
        return this._contents;
    }
    get pathDepth() {
        return this._root.row.path.length;
    }
    get sortStates() {
        return this._sortStates;
    }
    get root() {
        return this._root;
    }
    get options() {
        return { ...this._options };
    }
    set options(options) {
        const { columnNames, doRefetch = false, } = options;
        this._options = {
            columnNames: columnNames,
            doRefetch,
        };
    }
    get ixByColumn() {
        return this._ixByColumn;
    }
}
//# sourceMappingURL=model.js.map